package ssw.mj;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
 
/** 
 * <code>Compiler</code> is the driver for the MicroJava-Compiler.
 * 
 * Execute<br>
 * <code>java ssw.mj.Compiler &lt;<i>MJ-Source-Filename</i>&gt;</code><br>
 * to start compililation.
 */
public class Compiler {

	private static String objFileName(String s) {
		int i = s.lastIndexOf('.');
		if (i < 0) return s + ".obj";
		return s.substring(0, i) + ".obj";
	}
	
	public static void main(String[] args) {
		// --- get the filename
		if (args.length != 1) { 
			System.out.println(Messages.getString("USAGE")); 
			return;
		}
		String inFilename = args[0];
		String outFilename = objFileName(inFilename);
		
		try {
			
			Scanner scan = new Scanner(new BufferedReader(new FileReader(inFilename)));

			System.out.println("-----------------------------------");
			System.out.println(Messages.getString("PARSING", inFilename));
	
			Parser parser = new Parser(scan); 
			parser.parse();
			if (parser.scan.errors.numErrors() == 0) {
				parser.code.write(new BufferedOutputStream(new FileOutputStream(outFilename)));
			}
			
			if (parser.scan.errors.numErrors() > 0) {
				System.out.println(parser.scan.errors.dump());
				System.out.println(Messages.getString("ERRORS", parser.scan.errors.numErrors()));
			} else {
				System.out.println(Messages.getString("NO_ERRORS"));
			}
		} catch (IOException ex) {
			System.out.println(Messages.getString("FILE_ERROR", ex.getMessage()));
			return;
		}		
	}
}
