package ssw.mj.codegen;

import java.io.IOException;
import java.io.OutputStream;

import ssw.mj.Parser;

/**
 * MicroJava Code Generator.
 */
public class Code {
	// instruction codes
	public static final int
			load        =  1,
			load_0      =  2,
			load_1      =  3,
			load_2      =  4,
			load_3      =  5,
			store       =  6,
			store_0     =  7,
			store_1     =  8,
			store_2     =  9,
			store_3     = 10,
			getstatic   = 11,
			putstatic   = 12,
			getfield    = 13,
			putfield    = 14,
			const_0     = 15,
			const_1     = 16,
			const_2     = 17,
			const_3     = 18,
			const_4     = 19,
			const_5     = 20,
			const_m1    = 21,
			const_      = 22,
			add         = 23,
			sub         = 24,
			mul         = 25,
			div         = 26,
			rem         = 27,
			neg         = 28,
			shl         = 29,
			shr         = 30,
			inc         = 31,
			new_        = 32,
			newarray    = 33,
			aload       = 34,
			astore      = 35,
			baload      = 36,
			bastore     = 37,
			arraylength = 38,
			pop         = 39,
			dup         = 40,
			dup2        = 41,
			jmp         = 42,
			jeq         = 43,
			jne         = 44,
			jlt         = 45,
			jle         = 46,
			jgt         = 47,
			jge         = 48,
			call        = 49,
			return_     = 50,
			enter       = 51,
			exit        = 52,
			read        = 53,
			print       = 54,
			bread       = 55,
			bprint      = 56,
			trap        = 57;

	// codes for compare operators
	public static final int eq = 0, ne = 1, lt = 2, le = 3, gt = 4, ge = 5;

	private static final int[] inverse = { ne, eq, ge, gt, le, lt };

	/**	Code buffer */
	public byte[] buf;
	
	/**	Program counter.
	 *	Indicates next free byte in code buffer. 
	 */
	public int pc;

	/**	PC of main method (set by parser). */
	public int mainpc;

	/**	Length of static data in words (set by parser). */
	public int dataSize;
	
	/** According parser. */
	private Parser parser;

	//----- initialization

	public Code(Parser p) {
		parser = p;
		buf = new byte[100];
		pc = 0;
		mainpc = -1;
		dataSize = 0;
	}

	//----- code storage management

	public void put (int x) { 
		if (pc == buf.length) {
			byte[] nBuf = new byte[buf.length * 2];
			System.arraycopy(buf, 0, nBuf, 0, buf.length);
			buf = nBuf;
		}
		buf[pc++] = (byte) x; 
	}
	public void put2 (int x) { put(x>>8); put(x); }
	public void put4 (int x) { put2(x>>16); put2(x); }
	public void put2 (int pos, int x) {
		int oldpc = pc; pc = pos;
		put2(x);
		pc = oldpc;
	}

	public int get  (int pos) { return buf[pos]; }
	public int get2 (int pos) { 
		return (get(pos) << 8) + (get(pos+1) & 0xFF); 
	}


	// TODO Exercise 5 - 6: implementation of code generation 
	

	/** 
	 * Write the code buffer to the output stream. 
	 */
	public void write(OutputStream os) throws IOException {
		int codeSize = pc;
		// uncomment for debugging output
		// Decoder.decode(buf, 0, codeSize);
		put('M'); put('J');
		put4(codeSize);
		put4(dataSize);
		put4(mainpc);
		os.write(buf, codeSize, pc - codeSize);  // header
		os.write(buf, 0, codeSize);              // code
		os.flush();
		os.close();
	}
	
	/**
	 * String representation for JUnit test cases.
	 */
	public String dump() {
		StringBuilder sb = new StringBuilder();
		Decoder dec = new Decoder();
		sb.append(dec.decode(buf, 0, pc));
		sb.append("\n#CodeSize: ");
		sb.append(pc);
		sb.append("\n#DataSize: ");
		sb.append(dataSize);
		sb.append("\n#MainPC: ");
		sb.append(mainpc);
		return sb.toString();
	}
}
